'From Squeak 2.0 of May 22, 1998 on 3 November 1998 at 1:12:38 am'!
Object subclass: #StringParser
	instanceVariableNames: 'parser ocParser fastOutDetermination parserKeys ocParserKeys '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mathematics-TeX File Out'!
Object subclass: #TexFileOut
	instanceVariableNames: 'target fonts parser charsPerLine rightMargin currentWord commenting '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mathematics-TeX File Out'!

!ClassDescription methodsFor: 'Private - Tex fileOut' stamp: 'SqR!!!! 7/24/1998 17:49'!
texFileOutCategoriesUsing: texer
	"File out all the categories with their
	methods of the receiver. Use TeX format"

	self organization categories asSortedCollection do:
		[:each | self texFileOutCategory: each using: texer]! !

!ClassDescription methodsFor: 'Private - Tex fileOut' stamp: 'SqR!!!! 7/24/1998 17:47'!
texFileOutCategory: aCategory
	"File out a category of methods using TeX format"

	| fileStream |

	fileStream _ FileStream newFileNamed: self name , '-' , aCategory , '.tex'.
	self texFileOutCategory: aCategory using: (TexFileOut new: fileStream).
	fileStream close! !

!ClassDescription methodsFor: 'Private - Tex fileOut' stamp: 'SqR!!!! 7/24/1998 17:47'!
texFileOutCategory: aCategory using: texer
	"File out a category of methods using
	texer as the TeX formatter"

	texer outputCategory: aCategory.
	(self organization listAtCategoryNamed: aCategory) do:
		[:some | self texFileOutMethod: some using: texer]
! !

!ClassDescription methodsFor: 'Private - Tex fileOut' stamp: 'SqR!!!! 7/24/1998 17:43'!
texFileOutMethod: selector
	"File out the method for the message whose
	selector is selector. Create a file named as the
	selector plus '.tex'"

	| fileStream nameBody |

	(self includesSelector: selector) ifFalse: [^ self halt: 'Selector not found'].
	nameBody _ self name , '-' , (selector copyReplaceAll: ':' with: '').
	fileStream _ FileStream newFileNamed: nameBody , FileDirectory dot, 'tex'.
	self texFileOutMethod: selector using: (TexFileOut new: fileStream).
	fileStream close! !

!ClassDescription methodsFor: 'Private - Tex fileOut' stamp: 'SqR!!!! 7/24/1998 17:51'!
texFileOutMethod: selector using: texer
	"File out the method for the message whose
	selector is selector using texer as the TeX formatter"

	| methodSource |

	methodSource _ ReadWriteStream on: (String new: 32768).
	self printMethodChunk: selector withPreamble: false on: methodSource moveSource: false toFile: nil.
	methodSource reset.
	texer methodSource: methodSource
! !


!Class methodsFor: 'class name' stamp: 'SqR!!!!!!!!!!!!!!!! 3/28/98 19:42'!
rename: aString 
	"The new name of the receiver is the argument, aString."

	| newName |
	newName _ aString asSymbol.

"Fix renaming to current name"
	newName = self name ifTrue: [^self].

	(Smalltalk includesKey: newName)
		ifTrue: [^self error: newName , ' already exists'].
	(Undeclared includesKey: newName)
		ifTrue: [^ SelectionMenu notify: 'There are references to, ' , aString printString , '
from Undeclared. Check them after this change.'].
	Smalltalk renameClass: self as: newName.
	name _ newName.
	self comment: self comment.
	self class comment: self class comment
! !

!Class methodsFor: 'TeX fileOut' stamp: 'SqR!!!!!!!! 7/14/1998 22:15'!
texFileOut
	"File out the receiver into a file named as the
	receiver with extension 'tex'. Use TeX format"

	| fileStream |

	fileStream _ FileStream fileNamed: self class name asString, FileDirectory dot, 'tex'.
	self texFileOutOn: fileStream.
	fileStream close! !

!Class methodsFor: 'TeX fileOut' stamp: 'SqR!!!!!!!! 7/19/1998 22:01'!
texFileOutClassDefinitionOn: aStream
	"Output the definition of the receiver on
	aStream. Use TeX format"

	self texFileOutClassDefinitionUsing: (TexFileOut new: aStream)! !

!Class methodsFor: 'TeX fileOut' stamp: 'SqR!!!!!!!! 7/19/1998 21:57'!
texFileOutClassDefinitionUsing: texer
	"Output the definition of the receiver using
	texer as the TeX formatter"

	"Output class definition"
	texer 
		class: self name
		kind: self kindOfSubclass
		subclassOf: self superclass name
		category: (SystemOrganization categoryOfElement: self name)
		iVars: self instanceVariablesString
		cVars: self classVariablesString
		pDicts: self sharedPoolsString! !

!Class methodsFor: 'TeX fileOut' stamp: 'SqR!!!! 7/24/1998 04:15'!
texFileOutOn: aStream
	"File out the receiver into aStream. Use TeX format"

	self texFileOutUsing: (TexFileOut new: aStream)! !

!Class methodsFor: 'TeX fileOut' stamp: 'SqR!!!! 7/24/1998 17:50'!
texFileOutUsing: texer
	"File out the receiver. Use TeX format"
	"Shared Pools not supported yet"

	self texFileOutClassDefinitionUsing: texer.

	"Output class comments, if any"
	texer classComments: self organization classComment.

	"Continue with the metaclass"
	self class nonTrivial ifTrue: 
		[texer outputClassProtocol. self class texFileOutUsing: texer].

	"Now output the data for the class"
	methodDict size > 0 ifTrue:
		[
			texer outputInstanceProtocol.
			self texFileOutCategoriesUsing: texer
		]! !


!Metaclass reorganize!
('initialize-release' instanceVariableNames: newNamed: subclassOf:)
('accessing' isMeta isSystemDefined name soleInstance theNonMetaClass)
('copying' copy copyForValidation)
('instance creation' new)
('instance variables' addInstVarName: removeInstVarName:)
('pool variables' classPool)
('class hierarchy' name:inEnvironment:subclassOf:instanceVariableNames:variable:words:pointers:classVariableNames:poolDictionaries:category:comment:changed: subclasses subclassesDo:)
('compiling' acceptsLoggingOfCompilation possibleVariablesFor:continuedFrom: scopeHas:ifTrue: wantsChangeSetLogging)
('Private - TeX fileOut' texFileOutUsing:)
('fileIn/Out' definition fileOutOn:moveSource:toFile: nonTrivial objectToStoreOnDataStream storeDataOn:)
!


!Metaclass methodsFor: 'Private - TeX fileOut' stamp: 'SqR!!!! 7/24/1998 17:47'!
texFileOutUsing: texer
	"File out the receiver. Use TeX format"
	"Hyper alpha state"

	"Output the data for the class"
	self organization categories asSortedCollection do:
		[:each | self texFileOutCategory: each using: texer]! !


!Stream methodsFor: 'accessing' stamp: 'SqR!!!!!!!! 7/13/1998 02:51'!
nextLine
	"Answer the next line of text in the receiver"

	| result cr current |

	result _ OrderedCollection new.
	cr _ Character cr.
	current _ 0 asCharacter.
	[current = cr or: [self atEnd]] whileFalse:
		[current _ self next. result add: current].
	result add: current.
	(self atEnd not and: [self peek = Character linefeed]) 
		ifTrue: [result add: self next].
	^String newFrom: result! !


!PositionableStream methodsFor: 'accessing' stamp: 'SqR!!!!!!!! 7/14/1998 23:36'!
inclusiveUpTo: anObject 
	"Answer a subcollection from the current access position to the 
	occurrence (if any) of anObject in the receiver. If anObject is 
	not in the collection, answer the entire rest of the receiver."

	| newStream last |

	newStream _ WriteStream on: (collection species new: 100).
	[self atEnd or: [last = anObject]]
		whileFalse: [newStream nextPut: (last _ self next)].
	^newStream contents! !


!String methodsFor: 'Pascal stuff' stamp: 'SqR!!!!!!!! 6/7/1998 20:02'!
zapAll: aCharacter
	"Answer a copy of the receiver with all aCharacters removed"

	^self reject: [:char | char == aCharacter]! !

!String methodsFor: 'Pascal stuff' stamp: 'SqR!!!!!!!! 6/7/1998 20:02'!
zapAll: aCharacter1 and: aCharacter2
	"Answer a copy of the receiver with all aCharacter1 and aCharacter2 removed"

	^self reject: [:char | (char == aCharacter1) or: [char == aCharacter2]]! !

!String methodsFor: 'Pascal stuff' stamp: 'SqR!!!!!!!! 6/7/1998 20:09'!
zapCrAndLf
	"Answer a copy of the receiver without Cr and LF characters"

	^self zapAll: Character cr and: Character linefeed! !

!String methodsFor: 'Pascal stuff' stamp: 'SqR!!!!!!!! 6/7/1998 20:09'!
zapSpaces
	"Answer a copy of the receiver without space characters"

	^self zapAll: Character space! !

!String methodsFor: 'Pascal stuff' stamp: 'SqR!!!! 8/2/1998 17:33'!
zapTrailingSpaces
	"Answer a copy of the receiver without trailing space characters"

	self size to: 1 by: -1 do: 
		[:each |
			(self at: each) == Character space 
				ifFalse: [^self copyFrom: 1 to: each]
		].
	^''! !


!StringHolder methodsFor: 'message list menu' stamp: 'SqR!!!!!!!! 7/19/1998 21:14'!
texFileOutMessage
	"File out the selected message using tex format"

	self selectedMessageName ifNotNil: [
		self selectedClassOrMetaClass texFileOutMethod: self selectedMessageName]! !


!Browser methodsFor: 'system category functions' stamp: 'SqR!!!!!!!! 7/19/1998 21:18'!
systemCategoryMenu: aMenu

^ aMenu labels:
'find class...
recent classes...
browse all
browse
printOut
fileOut
reorganize
update
add item...
rename...
remove
TeX fileOut' 
	lines: #(2 4 6 8 11)
	selections:
		#(findClass recent browseAllClasses buildSystemCategoryBrowser
		printOutSystemCategory fileOutSystemCategory
		editSystemCategories updateSystemCategories
		addSystemCategory renameSystemCategory removeSystemCategory 
		texFileOutSystemCategory)
! !

!Browser methodsFor: 'system category functions' stamp: 'SqR!!!!!!!! 7/19/1998 21:41'!
texFileOutSystemCategory
	"Print a description of each class in the selected category onto a file 
	whose name is the category name followed by .tex. Use TeX format"

	Cursor write showWhile:
		[
			systemCategoryListIndex ~= 0
				ifTrue: [systemOrganizer texFileOutCategory: self selectedSystemCategoryName]
		]! !

!Browser methodsFor: 'class functions' stamp: 'SqR!!!!!!!! 7/19/1998 20:58'!
classListMenu: aMenu

^ aMenu labels: 
'browse class
browse full
printOut
fileOut
hierarchy
definition
comment
spawn hierarchy
spawn protocol
inst var refs..
inst var defs..
class var refs...
class vars
class refs
rename...
remove
unsent methods
find method...
TeX fileOut' 
	lines: #(4 7 9 11 14 16 18)
	selections:
		#(buildClassBrowser browseMethodFull printOutClass fileOutClass
		hierarchy editClass editComment
		spawnHierarchy spawnProtocol
		browseInstVarRefs browseInstVarDefs browseClassVarRefs 
		browseClassVariables browseClassRefs
		renameClass removeClass browseUnusedMethods findMethod
		texFileOutClass)
! !

!Browser methodsFor: 'class functions' stamp: 'SqR!!!!!!!! 7/19/1998 20:58'!
texFileOutClass
	"Print a description of the selected class onto a file whose name is the 
	category name followed by .tex. Use TeX format"

Cursor write showWhile:
		[classListIndex ~= 0 ifTrue: [self selectedClass texFileOut]]! !

!Browser methodsFor: 'message category functions' stamp: 'SqR!!!!!!!! 7/19/1998 21:50'!
messageCategoryMenu: aMenu

^ aMenu labels:
'browse
printOut
fileOut
reorganize
add item...
rename...
remove
TeX fileOut'
	lines: #(3 4 7)
	selections:
		#(buildMessageCategoryBrowser printOutMessageCategories fileOutMessageCategories
		editMessageCategories
		addCategory renameCategory removeMessageCategory
		texFileOutMessageCategories)
! !

!Browser methodsFor: 'message category functions' stamp: 'SqR!!!!!!!! 7/19/1998 21:50'!
texFileOutMessageCategories
	"Print a description of the selected message category of the selected class 
	onto an external file."

	Cursor write showWhile:
		[messageCategoryListIndex ~= 0
			ifTrue: 
				[self selectedClassOrMetaClass texFileOutCategory: self selectedMessageCategoryName]
		]! !

!Browser methodsFor: 'message functions' stamp: 'SqR!!!!!!!! 7/19/1998 21:02'!
messageListMenu: aMenu shifted: shifted
	^ shifted ifFalse: [aMenu labels:
'browse full
fileOut
printOut
senders of...
implementors of...
method inheritance
versions
inst var refs...
inst var defs...
class var refs...
class variables
class refs
remove
more...
TeX fileOut'
	lines: #(3 7 12 14)
	selections:
		#(browseMethodFull fileOutMessage printOutMessage
		browseSendersOfMessages browseMessages methodHierarchy browseVersions
		browseInstVarRefs browseInstVarDefs browseClassVarRefs 
			browseClassVariables browseClassRefs
		removeMessage shiftedYellowButtonActivity 
		texFileOutMessage)]

	ifTrue: [aMenu labels: 'browse class hierarchy
browse class
browse method
implementors of sent messages
change sets with this method
inspect instances
inspect subinstances
remove from this browser
revert to previous version
remove from current change set
revert and forget
more...' 
	lines: #(5 7 11)
	selections: #(classHierarchy browseClass 
		buildMessageBrowser browseAllMessages findMethodInChangeSets 
		inspectInstances inspectSubInstances
		removeMessageFromBrowser revertToPreviousVersion 
		removeFromCurrentChanges revertAndForget
		unshiftedYellowButtonActivity)]
! !


!StringParser reorganize!
('Accessing' parseKeyAmount)
('Parser loading' doNotParse: loadDictionary: parse:as: parsedFor: parses: unloadDictionary:)
('Parser loading (open close)' doNotocParse: ocParses: parse:whenOpenAs:whenClosedAs:)
('Parsing' initializeOpenClosed parse:)
('Printing' printOn:)
('Private' initialize)
('Private - Enumerating' allParserKeysDo:)
('Private - Fast Out Determination' fastOutFor: rebuildFastOut updateFastOut:)
('Private - Parser manteinance' ensureSubParser: ensureSubocParser: ensureUsefulSubParser: ensureUsefulSubocParser:)
('Private - Parsing' parse:at:)
!


!StringParser methodsFor: 'Accessing' stamp: 'SqR!!!! 8/2/1998 03:27'!
parseKeyAmount
	"Answer the amount of parsing keys"

	^(parser inject: 0 into: [:total :each | total + each size]) +
		(ocParser inject: 0 into: [:total2 :each2 | total2 + each2 size])! !

!StringParser methodsFor: 'Parser loading' stamp: 'SqR!!!! 8/2/1998 02:55'!
doNotParse: aSubstring
	"Leave occurrences of aSubstring alone"

	(parserKeys includes: aSubstring size) ifTrue:
		[
			(parser at: aSubstring size) removeKey: aSubstring ifAbsent: [].
			self ensureUsefulSubParser: aSubstring size.
			self rebuildFastOut.
		]! !

!StringParser methodsFor: 'Parser loading' stamp: 'SqR!!!! 8/2/1998 20:43'!
loadDictionary: aDictionary
	"Parse every key in aDictionary as its corresponding value"

	aDictionary associationsDo:
		[:each | self parse: each key as: each value]! !

!StringParser methodsFor: 'Parser loading' stamp: 'SqR!!!! 8/2/1998 02:25'!
parse: aKey as: aReplacement
	"Set the parser to parse aKey as aReplacement"

	self ensureSubParser: aKey size.
	(parser at: aKey size) at: aKey put: aReplacement.
	self updateFastOut: aKey! !

!StringParser methodsFor: 'Parser loading' stamp: 'SqR!!!! 8/2/1998 02:13'!
parsedFor: aSubstring
	"Answer the replacement the receiver
	uses for aSubstring. If no replacement
	is used, answer nil"

	(parserKeys includes: aSubstring size) ifFalse: [^nil].
	^(parser at: aSubstring size) at: aSubstring ifAbsent: []! !

!StringParser methodsFor: 'Parser loading' stamp: 'SqR!!!! 8/2/1998 02:13'!
parses: aSubstring
	"Answer true if the receiver parses aSubstring"

	^(self parsedFor: aSubstring) isNil not! !

!StringParser methodsFor: 'Parser loading' stamp: 'SqR!!!! 8/2/1998 02:15'!
unloadDictionary: aDictionary
	"Do not parse any key in aDictionary"

	aDictionary associationsDo: [:each | self doNotParse: each key]! !

!StringParser methodsFor: 'Parser loading (open close)' stamp: 'SqR!!!! 8/2/1998 02:56'!
doNotocParse: aSubstring
	"Leave occurrences of aSubstring alone"

	(ocParserKeys includes: aSubstring size) ifTrue:
		[
			(ocParser at: aSubstring size) removeKey: aSubstring ifAbsent: [].
			self ensureUsefulSubocParser: aSubstring size.
			self rebuildFastOut.
		]! !

!StringParser methodsFor: 'Parser loading (open close)' stamp: 'SqR!!!! 8/2/1998 03:30'!
ocParses: aSubstring
	"Answer true if the receiver 
	open closed parses aSubstring"

	(ocParserKeys includes: aSubstring size) ifFalse: [^false].
	^(ocParser at: aSubstring size) includesKey: aSubstring! !

!StringParser methodsFor: 'Parser loading (open close)' stamp: 'SqR!!!! 8/2/1998 03:25'!
parse: aKey whenOpenAs: openRep whenClosedAs: closedRep
	"Parse aKey as openRep when it's open (odd occurrence)
	and as closedRep when it's closed (even occurrence)"

	self ensureSubocParser: aKey size.
	(ocParser at: aKey size)
		at: aKey
		put: (Array with: 2 with: openRep with: closedRep).
	self updateFastOut: aKey! !

!StringParser methodsFor: 'Parsing' stamp: 'SqR!!!! 8/2/1998 20:59'!
initializeOpenClosed
	"Initialize the open closed parsing"

	ocParserKeys do:
		[:each |
			(ocParser at: each) do:
				[:some | some at: 1 put: 2]
		]! !

!StringParser methodsFor: 'Parsing' stamp: 'SqR!!!! 8/2/1998 20:59'!
parse: aString
	"Answer the parsed string for aString"

	| index answer currentParsed skip aStringSize temp |

	answer _ OrderedCollection new: (aString size * 1.1) truncated.
	index _ 1.
	aStringSize _ aString size.
	[index > aStringSize] whileFalse:
		[
			(self fastOutFor: (aString at: index))
				ifTrue: [answer add: (aString at: index). index _ index + 1]
				ifFalse: 
					[
						currentParsed _ nil.
						skip _ parserKeys detect: 
							[:one |
								one + index - 1 > aStringSize ifFalse:
									[currentParsed _ (parser at: one) at: (aString copyFrom: index to: index + one - 1)
										ifAbsent: []].
								currentParsed isNil not
							] ifNone: 
							[
								ocParserKeys detect:
									[:another |
										another + index - 1 > aStringSize ifFalse:
											[
												temp _ (ocParser at: another) at: (aString copyFrom: index to: index + another - 1)
													ifAbsent: [].
												temp isNil ifFalse:
													[
														currentParsed _ temp at: (temp at: 1).
														temp at: 1 put: (temp at: 1) - 1 \\ 2 + 2.
													].
											].
										currentParsed isNil not
									] ifNone: []
							].
						skip isNil ifTrue: [answer add: (aString at: index). index _ index + 1]
							ifFalse: [answer addAll: currentParsed. index _ index + skip].
					].
		].
	^String newFrom: answer! !

!StringParser methodsFor: 'Printing' stamp: 'SqR!!!! 8/2/1998 02:22'!
printOn: aStream
	"Blah!!"

	| keys |

	aStream nextPutAll: self class name.
	aStream nextPut: $(.
	keys _ self parseKeyAmount.
	keys printOn: aStream.
	keys = 1 
		ifTrue: [aStream nextPutAll: ' key)']
		ifFalse: [aStream nextPutAll: ' keys)']! !

!StringParser methodsFor: 'Private' stamp: 'SqR!!!! 8/2/1998 02:49'!
initialize
	"Initialize the receiver"

	parser _ Dictionary new.
	parserKeys _ SortedCollection sortBlock: [:one :another | another < one].
	ocParser _ Dictionary new.
	ocParserKeys _ SortedCollection sortBlock: [:one :another | another < one].
	self rebuildFastOut! !

!StringParser methodsFor: 'Private - Enumerating' stamp: 'SqR!!!! 8/2/1998 02:49'!
allParserKeysDo: aBlock
	"Evaluate aBlock for all the keys of the parser"

	parser do: [:each | each keys do: aBlock].
	ocParser do: [:each | each keys do: aBlock].! !

!StringParser methodsFor: 'Private - Fast Out Determination' stamp: 'SqR!!!! 8/2/1998 02:33'!
fastOutFor: aCharacter
	"Answer if a fast out can be
	performed on aCharacter"

	^fastOutDetermination at: aCharacter asciiValue + 1! !

!StringParser methodsFor: 'Private - Fast Out Determination' stamp: 'SqR!!!! 8/2/1998 17:10'!
rebuildFastOut
	"Rebuild the fast out determination cache"

	fastOutDetermination _ Array new: 256.
	fastOutDetermination atAllPut: true.
	self allParserKeysDo: [:each | self updateFastOut: each]! !

!StringParser methodsFor: 'Private - Fast Out Determination' stamp: 'SqR!!!! 8/2/1998 17:10'!
updateFastOut: aKey
	"Update the fast out determination cache"

	fastOutDetermination at: (aKey at: 1) asciiValue + 1 put: false! !

!StringParser methodsFor: 'Private - Parser manteinance' stamp: 'SqR!!!! 8/2/1998 02:11'!
ensureSubParser: anInteger
	"Ensure the sub parser for anInteger character
	parsing keys exists and is operational"

	(parserKeys includes: anInteger) ifFalse: 
		[
			parser at: anInteger put: Dictionary new.
			parserKeys add: anInteger
		]! !

!StringParser methodsFor: 'Private - Parser manteinance' stamp: 'SqR!!!! 8/2/1998 02:50'!
ensureSubocParser: anInteger
	"Ensure the sub ocParser for anInteger character
	parsing keys exists and is operational"

	(ocParserKeys includes: anInteger) ifFalse: 
		[
			ocParser at: anInteger put: Dictionary new.
			ocParserKeys add: anInteger
		]! !

!StringParser methodsFor: 'Private - Parser manteinance' stamp: 'SqR!!!! 8/2/1998 02:19'!
ensureUsefulSubParser: anInteger
	"Ensure the sub parser for anInteger character
	parsing keys is useful. Otherwise, delete it"

	((parserKeys includes: anInteger) and: 
		[(parser at: anInteger) size = 0]) ifTrue:
			[
				parser removeKey: anInteger.
				parserKeys removeAt: (parserKeys indexOf: anInteger)
			]! !

!StringParser methodsFor: 'Private - Parser manteinance' stamp: 'SqR!!!! 8/2/1998 02:51'!
ensureUsefulSubocParser: anInteger
	"Ensure the sub ocParser for anInteger character
	parsing keys is useful. Otherwise, delete it"

	((ocParserKeys includes: anInteger) and: 
		[(ocParser at: anInteger) size = 0]) ifTrue:
			[
				ocParser removeKey: anInteger.
				ocParserKeys removeAt: (ocParserKeys indexOf: anInteger)
			]! !

!StringParser methodsFor: 'Private - Parsing' stamp: 'SqR!!!! 8/2/1998 03:03'!
parse: aString at: anIndex
	"Answer the parsing of aString at anIndex"

	| answer |

	parserKeys do:
		[:each |
			each + anIndex - 1 > aString size ifFalse:
				[
					answer _ (parser at: each) at: (aString copyFrom: anIndex to: anIndex + each - 1).
					answer isNil ifFalse: [^answer].
				].
		].
	ocParserKeys do:
		[:each |
			each + anIndex - 1 > aString size ifFalse:
				[
					answer _ (ocParser at: each) at: (aString copyFrom: anIndex to: anIndex + each - 1).
					answer isNil ifFalse: [^answer].
				].
		].
	^(aString at: anIndex) asString! !


!StringParser class reorganize!
('Instance creation' new)
!


!StringParser class methodsFor: 'Instance creation' stamp: 'SqR!!!! 8/2/1998 01:37'!
new
	"Answer a new instance of the receiver"

	^super new initialize! !


!SystemOrganizer reorganize!
('fileIn/Out' fileOutCategory: fileOutCategory:asHtml: fileOutCategory:on: superclassOrder:)
('remove' removeCategoriesMatching: removeMissingClasses removeSystemCategory:)
('TeX fileOut' texFileOutCategory: texFileOutCategory:using:)
!


!SystemOrganizer methodsFor: 'TeX fileOut' stamp: 'SqR!!!! 7/24/1998 04:14'!
texFileOutCategory: category 
	"Store on the file named category (a string) concatenated with '.tex' all the 
	classes associated with the category. Use TeX format"

	| fileStream |

	fileStream _ FileStream fileNamed: category, FileDirectory dot, 'tex'.
	self texFileOutCategory: category using: (TexFileOut new: fileStream).
	fileStream close! !

!SystemOrganizer methodsFor: 'TeX fileOut' stamp: 'SqR!!!!!!!! 7/19/1998 21:42'!
texFileOutCategory: category using: texer
	"Fileout all the classes associated with the category. 
	Use texer as the TeX formatter.

	Does not support shared pools yet."

	Cursor write showWhile:
		[(self superclassOrder: category) do: [:class | class texFileOutUsing: texer]]! !


!TexFileOut reorganize!
('File out' class:kind:subclassOf:category:iVars:cVars:pDicts: classComments: methodSource: outputCategory: outputClassProtocol outputInstanceProtocol)
('File out fonts' fontFor: setDefaultFonts setFontFor:to:)
('Presets' replaceTabWith: rightMargin rightMargin: tabReplacement)
('TeX control' beginSection bigSkip closeBlock closeItemVBox closeLine commenting: crLf emptyItem: endSection medSkip noIndent openFontBlock: openFontBlockWithFontFor: openItemVBox openLine setFontTo: smallSkip terminateLine)
('Private - Initializing' initialize: initializeParser stampVersion)
('Private - Method file out' methodCodeLine: methodEndLine methodSelector:)
('Private - Text parsing' ?!! nextPut: nextPutAll: nextPutComments: nextPutLine: tryToFlush:)
!


!TexFileOut methodsFor: 'File out' stamp: 'SqR!!!! 7/24/1998 18:14'!
class: className kind: kindOfSubclass subclassOf: superclassName category: cat iVars: iVars cVars: cVars pDicts: pDicts
	"Output a class definition with the given parameters"

	"Open section"
	self nextPutComments: 'Class definition submitted for class: ', className, '.'.
	self openFontBlockWithFontFor: #normal.
	self crLf.
	self beginSection.

	"First line containing class name"
	self openFontBlockWithFontFor: #bold.
	self nextPutAll: superclassName, kindOfSubclass.
	self closeBlock.
	self nextPutLine: className.

	"Subsequent lines containing class data"
	self crLf.
	cat isEmpty ifFalse: [self emptyItem: 'Category: ', cat].
	iVars isEmpty ifFalse: [self emptyItem: 'Instance variable names: ', iVars].
	cVars isEmpty ifFalse: [self emptyItem: 'Class variable names: ', cVars].
	pDicts isEmpty ifFalse: [self emptyItem: 'Shared pool dictionaries: ', pDicts].

	self closeBlock.
	self endSection.
	self tryToFlush: true! !

!TexFileOut methodsFor: 'File out' stamp: 'SqR!!!! 8/2/1998 21:01'!
classComments: aString
	"Output class comments in aString"

	aString isEmpty ifTrue: [^self].
	self nextPutComments: 'Comments submitted.'.
	parser initializeOpenClosed.
	self openFontBlockWithFontFor: #comments.
	self beginSection.
	self nextPutLine: self ?!! aString.
	self closeBlock.
	self endSection.
	self tryToFlush: true! !

!TexFileOut methodsFor: 'File out' stamp: 'SqR!!!! 8/2/1998 21:00'!
methodSource: aStream
	"Output the method source in aStream"

	| currentLine |

	self nextPutComments: 'Method submitted.'.
	parser initializeOpenClosed.
	self openFontBlockWithFontFor: #normal.
	self crLf.
	self bigSkip.
	currentLine _ aStream nextLine.
	self methodSelector: currentLine zapCrAndLf.
	[aStream atEnd] whileFalse:
		[
			currentLine _ aStream nextLine zapCrAndLf. 
			aStream atEnd ifTrue: 
				[currentLine _ currentLine copyFrom: 1 to: currentLine size - 1].
			self methodCodeLine: currentLine
		].
	self closeBlock.
	self endSection.
	self tryToFlush: true! !

!TexFileOut methodsFor: 'File out' stamp: 'SqR!!!!!!!! 7/14/1998 23:04'!
outputCategory: aString
	"Output a category banner"

	self bigSkip smallSkip noIndent.
	self openFontBlockWithFontFor: #bold.
	self nextPutAll: 'Methods for category: ', aString.
	self closeBlock.
	self crLf.! !

!TexFileOut methodsFor: 'File out' stamp: 'SqR!!!!!!!! 7/14/1998 23:04'!
outputClassProtocol
	"Output a class protocol banner"

	self bigSkip medSkip noIndent.
	self openFontBlockWithFontFor: #bold.
	self nextPutAll: 'Class protocol'.
	self closeBlock.
	self crLf.! !

!TexFileOut methodsFor: 'File out' stamp: 'SqR!!!!!!!! 7/14/1998 23:04'!
outputInstanceProtocol
	"Output an instance protocol banner"

	self bigSkip medSkip noIndent.
	self openFontBlockWithFontFor: #bold.
	self nextPutAll: 'Instance protocol'.
	self closeBlock.
	self crLf.! !

!TexFileOut methodsFor: 'File out fonts' stamp: 'SqR!!!!!!!! 7/13/1998 01:04'!
fontFor: aSymbol
	"Answer the font corresponding to aSymbol"

	^fonts at: aSymbol! !

!TexFileOut methodsFor: 'File out fonts' stamp: 'SqR!!!!!!!! 7/13/1998 01:34'!
setDefaultFonts
	"Set the default fonts"

	fonts at: #bold put: 'ssbf'.
	fonts at: #normal put: 'ss'.
	fonts at: #italic put: 'ssit'.
	fonts at: #comments put: 'rm'! !

!TexFileOut methodsFor: 'File out fonts' stamp: 'SqR!!!!!!!! 7/13/1998 00:13'!
setFontFor: aSymbol to: aString
	"Set the font corresponding to aSymbol to aString.
	Please do not add useless fonts"

	(fonts includesKey: aSymbol) ifFalse: [^self error: 'Useless font'].
	fonts at: aSymbol put: aString! !

!TexFileOut methodsFor: 'Presets' stamp: 'SqR!!!! 7/24/1998 04:16'!
replaceTabWith: aString
	"Replace the tab characters with
	the TeX expression contained in aString"

	(parser at: 1) at: Character tab asString put: aString! !

!TexFileOut methodsFor: 'Presets' stamp: 'SqR!!!!!!!! 7/14/1998 23:06'!
rightMargin
	"Answer the right margin for outputting a tex source"

	^rightMargin! !

!TexFileOut methodsFor: 'Presets' stamp: 'SqR!!!!!!!! 7/14/1998 23:06'!
rightMargin: anInteger
	"Set the right margin for outputting 
	a tex source to anInteger characters"

	rightMargin _ anInteger! !

!TexFileOut methodsFor: 'Presets' stamp: 'SqR!!!! 7/24/1998 04:16'!
tabReplacement
	"Answer the current tab replacement"

	^(parser at: 1) at: Character tab asString! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 03:28'!
beginSection
	"Begin a section"

	self nextPutLine: '\beginsection'.
	self crLf! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 03:28'!
bigSkip
	"Output a bigskip"

	self crLf.
	self nextPutLine: '\bigskip'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:01'!
closeBlock
	"Close a block"

	self nextPutAll: '}'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:01'!
closeItemVBox
	"Close an item vbox"

	self nextPutAll: '}$$'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:01'!
closeLine
	"Close a line"

	self closeBlock crLf! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 18:13'!
commenting: aBoolean
	"Set the commenting feature on or off"

	commenting _ aBoolean! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 18:48'!
crLf
	"Output a cr lf pair"

	self tryToFlush: true.
	self terminateLine! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 03:29'!
emptyItem: aString
	"Append an empty item"

	self nextPutLine: '\item{} ', aString! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/13/1998 01:36'!
endSection
	"End a section"

	self crLf crLf! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 03:29'!
medSkip
	"Output a medskip"

	self crLf.
	self nextPutLine: '\medskip'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 17:58'!
noIndent
	"Output a noIndent"

	self nextPutAll: '\noindent '! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:03'!
openFontBlock: aFontName
	"Open a font block"

	self nextPutAll: '{\', aFontName, ' '! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:03'!
openFontBlockWithFontFor: aSymbol
	"Open a font block"

	self openFontBlock: (self fontFor: aSymbol)! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:03'!
openItemVBox
	"Open an item vbox"

	self nextPutAll: '$$\vbox{'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:03'!
openLine
	"Open a line"

	self nextPutAll: '\line{'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!!!!!! 7/14/1998 23:03'!
setFontTo: aSymbol
	"Set the font to the one stored for aSymbol"

	self nextPutAll: '\', (self fontFor: aSymbol)! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 03:29'!
smallSkip
	"Output a smallskip"

	self crLf.
	self nextPutLine: '\smallskip'! !

!TexFileOut methodsFor: 'TeX control' stamp: 'SqR!!!! 7/24/1998 18:48'!
terminateLine
	"Terminate the current line of output
	and update all stuff to process next line"

	target cr lf.
	charsPerLine _ 0
! !

!TexFileOut methodsFor: 'Private - Initializing' stamp: 'SqR!!!! 7/24/1998 19:01'!
initialize: aStream
	"Initialize the receiver"

	fonts _ Dictionary new.
	self setDefaultFonts.
	charsPerLine _ 0.
	rightMargin _ 72.
	currentWord _ String new.
	target _ aStream.
	commenting _ false.
	self initializeParser.
	self stampVersion! !

!TexFileOut methodsFor: 'Private - Initializing' stamp: 'SqR!!!! 8/2/1998 20:50'!
initializeParser
	"Initialize the parser"

	| parser1 parser2 parser3 |

	parser1 _ Dictionary new.
	parser1 at: '_' put: '$\leftarrow$'.
	parser1 at: '^' put: '$\uparrow$'.
	parser1 at: '\' put: '$\backslash$'.
	parser1 at: '$' put: '\$'.
	parser1 at: '%' put: '\%'.
	parser1 at: '{' asString put: '$\{$'.
	parser1 at: '}' asString put: '$\}$'.
	parser1 at: '&' put: '\&'.
	parser1 at: '#' put: '\#'.
	parser1 at: '|' put: '$|$'.
	parser1 at: '>' put: '$>$'.
	parser1 at: '<' put: '$<$'.
	parser1 at: '-' put: '$-$'.
	parser1 at: '+' put: '$+$'.
	parser1 at: '*' put: '$\times$'.
	parser1 at: '/' put: '$/$'.
	parser1 at: '~' put: '$\sim$'. "\~{}"
	parser1 at: '=' put: '$=$'.
	parser1 at: Character tab asString put: '\quad '.

	parser2 _ Dictionary new.
	parser2 at: '>=' put: '$\geq$'.
	parser2 at: '<=' put: '$\leq$'.
	parser2 at: '~=' put: '$\not =$'.
	parser2 at: '[]' put: '[$\,$]'.
	parser2 at: ']]' put: ']$\,$]'.
	parser2 at: '!!!!' put: '!!'.
	parser2 at: '''''' put: '''$\,$'''.
	parser2 at: '->' put: '$\rightarrow$'.

	parser3 _ Dictionary new.
	parser3 at: '[]]' put: '[$\,$]$\,$]'.

	parser _ StringParser new.
	parser loadDictionary: parser1.
	parser loadDictionary: parser2.
	parser loadDictionary: parser3.
	parser parse: '"' whenOpenAs: '``' whenClosedAs: '"'! !

!TexFileOut methodsFor: 'Private - Initializing' stamp: 'SqR!!!! 8/2/1998 17:16'!
stampVersion
	"Stamp a version notice on the fileout"

	self nextPutComments: 'TeX fileOut version 19980802 by SqR!!'! !

!TexFileOut methodsFor: 'Private - Method file out' stamp: 'SqR!!!! 7/24/1998 04:17'!
methodCodeLine: aString
	"Output a method code line"

	self noIndent.
	(aString detect: [:one | one asciiValue > 32] ifNone: [])
		isNil ifTrue: [self medSkip] ifFalse: [self nextPutAll: self ?!! aString. self methodEndLine]! !

!TexFileOut methodsFor: 'Private - Method file out' stamp: 'SqR!!!! 7/24/1998 03:58'!
methodEndLine
	"End a method line"

	self crLf crLf! !

!TexFileOut methodsFor: 'Private - Method file out' stamp: 'SqR!!!! 8/2/1998 17:34'!
methodSelector: aString
	"Output a method selector"

	| data bold words |

	self noIndent.
	data _ ReadStream on: aString.
	bold _ true.
	words _ 0.
	[data atEnd] whileFalse:
		[
			bold 
				ifTrue: [self openFontBlockWithFontFor: #bold]
				ifFalse: [self openFontBlockWithFontFor: #italic].
			self nextPutAll: (self ?!! (data nextDelimited: Character space) zapTrailingSpaces).
			(data atEnd or: [bold]) ifFalse: [self nextPutAll: '\/ '].
			self closeBlock.
			(data atEnd not and: [bold]) ifTrue: [self nextPut: $~].
			bold _ bold not.
			words _ words + 1.
			words \\ 8 = 0 ifTrue: [self crLf" crLf. self noIndent"].
		].
	self methodEndLine! !

!TexFileOut methodsFor: 'Private - Text parsing' stamp: 'SqR!!!! 8/2/1998 17:13'!
?!! aString
	"Answer aString parsed for TeX"

	^parser parse: aString! !

!TexFileOut methodsFor: 'Private - Text parsing' stamp: 'SqR!!!!!!!! 7/14/1998 23:54'!
nextPut: aCharacter
	"Output a character"

	currentWord _ currentWord, aCharacter asString.
	self tryToFlush: false! !

!TexFileOut methodsFor: 'Private - Text parsing' stamp: 'SqR!!!!!!!! 7/14/1998 23:59'!
nextPutAll: aString
	"Output a string"

	| stream |

	stream _ ReadStream on: aString.
	[stream atEnd] whileFalse:
		[
			currentWord _ currentWord, (stream inclusiveUpTo: Character space).
			self tryToFlush: false
		]! !

!TexFileOut methodsFor: 'Private - Text parsing' stamp: 'SqR!!!! 7/24/1998 18:59'!
nextPutComments: aString
	"Output aString followed by cr and lf.
	Add comment strings so that TeX ignores it"

	self commenting: true.
	self nextPutAll: '% ', aString.
	self crLf.
	self commenting: false! !

!TexFileOut methodsFor: 'Private - Text parsing' stamp: 'SqR!!!! 7/24/1998 03:27'!
nextPutLine: aString
	"Output aString followed by cr and lf"

	self nextPutAll: aString.
	self crLf! !

!TexFileOut methodsFor: 'Private - Text parsing' stamp: 'SqR!!!! 8/2/1998 17:34'!
tryToFlush: forced
	"Attempt to flush the current word"

	| toWrite overflow |

	overflow _ charsPerLine + 1 + currentWord size > rightMargin.
	(currentWord size ~= 0 and: [(currentWord last = Character space or: [forced])])
		ifTrue:
			[
				overflow ifTrue: [self terminateLine]. 
				charsPerLine = 0
					ifTrue: [toWrite _ currentWord zapTrailingSpaces]
					ifFalse: [toWrite _ Character space asString, currentWord zapTrailingSpaces].
				commenting & overflow ifTrue: [toWrite _ '% ', toWrite].
				target nextPutAll: toWrite.
				charsPerLine _ charsPerLine + toWrite size.
				currentWord _ ''
			]! !


!TexFileOut class reorganize!
('Instance creation' new:)
!


!TexFileOut class methodsFor: 'Instance creation' stamp: 'SqR!!!!!!!! 7/12/1998 23:52'!
new: aStream
	"Answer a new instance of the receiver"

	^self new initialize: aStream! !


!WriteStream methodsFor: 'character writing' stamp: 'SqR!!!!!!!! 7/13/1998 00:58'!
lf
	"Append a line feed character to the receiver."

	self nextPut: Character linefeed! !


ClassDescription removeSelector: #texFileOutOn:using:!
Class removeSelector: #texFileOutOn:using:!
Class removeSelector: #texFileOutMessage:using:!
Class removeSelector: #texFileOutCategory:using:!
Class removeSelector: #texFileOutOnUsing:!
Class removeSelector: #texFileOutCategory:!
Class removeSelector: #texFileOutMethod:!
Class removeSelector: #texFileOutMethod:using:!
Metaclass removeSelector: #texFileOutOn:using:!
Metaclass removeSelector: #texFileOutCategoriesUsing:!
Metaclass removeSelector: #texFileOutOnUsing:!
StringParser removeSelector: #new!
StringParser removeSelector: #rebuildFastOutDetermination!
TexFileOut removeSelector: #closeFontBlock!
TexFileOut removeSelector: #closeSmalltalk!
TexFileOut removeSelector: #openSmalltalk!
TexFileOut removeSelector: #class:subclassOf:category:iVars:cVars:pDicts:!
TexFileOut removeSelector: #??!
TexFileOut removeSelector: #initSmalltalk!
TexFileOut removeSelector: #startup!
TexFileOut removeSelector: #shutdown!
TexFileOut removeSelector: #writeOutLine:!
TexFileOut removeSelector: #parse:at:!
